/*
**             LIMED
**
**             Version 1.0
**
**             Limiting median hyperplanes for two- and
**             three-dimensional k-cameral legislatures
**
**             (c) 2002 by Thomas Braeuninger, University of Konstanz,
**             Germany; email: thomas.braeuninger@uni-konstanz.de
**
**
**             citation:
**             Thomas Braeuninger (2003) 'When simple voting doesn't work. Multi-
**             cameral systems for the representation and aggregation of interests   
**             in international organizations', British Journal of Political 
**             Science 33(1): 107-131.
**
**
** Purpose:    LIMED computes limiting median hyperplanes in two- and three-
**             dimensional spatial models for uni- and multi-cameral legislatures 
**             
**             
**
** Input:      dataset of actors' ideal points
**                m x (k+1) matrix
**                column 1:              actor number 1,...,m
**                columns 2 to (k+1):    ideal points in 2 or 3 dimensions
**
**             - for unicameral legislatures -
**             dataset of voting weights
**                m x (1+k) matrix
**                column 1:              actor number 1,...,m
**                columns 2 to (k+1):    actor's voting weights
**
**             - for multicameral legislatures -
**             dataset of chamber composition
**                m x 2 matrix
**                column 1:              actor number 1,...,m
**                column 2:              actor's cameral affiliation
**             dataset of voting weights
**                m x 2 matrix
**                column 1:              actor number 1,...,m
**                columns 2 to (k+1):    actor's voting weights
**
**
** Output:     .log file
**             input data summary and limiting median hyperplanes 
**             in 2- or 3-point form and in vector-offset form
**
**             .h file
**             limiting median hyperplanes in vector-offset form
**
**             .bat
**             command file for QHULL
*/




new;
goto mainproc;




/*
** loaddat - loads input data from file
*/

loaddat:

   "Select type of problem to be analysed:";
   "(1) Unicameral legislature with actors having one or multiple voting weights";
   "(2) Multi-cameral legislature with actors having (un-)weighted votes";
   typ=con(1,1);
   screen off; 
   typ;
   screen on;
   typ=typ-1;
   " ";
   "Total number of actors: ";
   m=con(1,1);
   screen off; 
   m;
   screen on;
   "Data file containing ideal points: ";
   name1=cons;
   name1;
   load x[] = ^name1;
   dim=(rows(x)/m)-1;
   "Policy dimensions: ";
   dim;
   x1=reshape(x,m,dim+1);
   x=x1[.,2:1+dim];
   if dim<3;
     x=x~zeros(m,1);
     dim1=dim+1;
   else;
     dim1=dim;
   endif;
   if typ==0;
     "Number of voting weights:";
     gew=con(1,1);
     screen off; 
     gew;
     screen on;
     name4=" ";
   else;
     "Data file containing chamber assignment: ";
     name4=cons;
     name4;
     load k1[m,2] = ^name4;
     k=k1[.,2];
     kcameral=maxc(k);
     gew=1;
   endif;
   "Data file containing voting weights: ";
   name2=cons;
   name2;
   " ";
   load g1[m,1+gew] = ^name2;
   g=g1[.,2:gew+1];

   if typ==0;
     thres=zeros(1,gew);
     "Sum of voting weights (for each weight system): ";
     i=0;
     do until i==gew;
       i=i+1;
       "weight system "
       i
       ": "
       sumc(g[.,i]);
     endo;
     "Voting thresholds (one threshold per weight system): ";
     thres=con(1,gew);
     screen off; 
     thres;
     screen on;
     " ";
     T=0;
   else;
     "Sum of voting weights (for each chamber): ";
     i=0;
     do until i==kcameral;
       i=i+1;
       "chamber "
       i
       ": "
       sumc(selif(g,(k.==i)));
     endo;
     "Voting thresholds (one threshold per chamber):";
     thres=con(1,kcameral);
     screen off; 
     thres;
     screen on;
     "Sum of voting weights (over all chambers): ";
     sumc(g);
     "Voting threshold (over all chambers):";
     T=con(1,1);
     screen off; 
     T;
     screen on;
     " ";
   endif;

   "Name of output file (without extension): ";
   name3 = cons;
   name3;
   "Write command file for QHULL? (y/n): ";
   qhull = cons;
   qhull;
   format /rd 5,2;
   if (lower(qhull) $== "y");
     "Starting point: ";
     miny=con(dim,1);
     screen off; 
     miny';
     screen on;
     "Range: ";
     rangey=con(dim,1);
     screen off; 
     rangey';
     screen on;
     "Inkrement: ";
     eta=con(1,1);
     if eta <= 0;
       eta = 1;
     endif;
     screen off; 
     eta;
     screen on;
   endif;
   " ";

return;




/*
** outlog - writes output to logfile
*/

outlog:

   outwidth 256;
   format /rd 5,0;
   name5 = "limed01.log";
   output file = ^name5 reset;

return;





/*
** outqhull - produces command file for QHULL
*/

outqhull:

   Hplane=Hplane[.,1:dim cols(Hplane)];
   if rows(Hplane) > 2;
     Hplane = Hplane[2:rows(Hplane),.];
   endif;
   name6 = name3 $+ ".h";
   output file = ^name6 reset;
   screen off;
   format /rd 10,0;
   dim+1;
   rows(Hplane);
   format /rd 20,16;
   Hplane;
   output off;
   name6 = name3 $+ ".bat";
   output file = ^name6 reset;
   "REM input file for Qhull";
   "REM run from DOS";
   if dim==2;
     rangey=rangey|zeros(1,1);
     miny=miny|zeros(1,1);
   endif;
   rangey=miny+rangey;
   point=miny;
   do until point[1]>rangey[1];
     point[2]=miny[2];
     do until point[2]>rangey[2];
       point[3]=miny[3];
       do until point[3]>rangey[3];
         point1 = ftos(point[1],"%*.*lf",1,4);
         point2 = ftos(point[2],"%*.*lf",1,4);
         if dim==3;
           point3 = ftos(point[3],"%*.*lf",1,4);
         endif;
         text1 = "qhalf s Fp H" $+ point1 $+ "," $+ point2 ;
         if dim==3;
           text1 = text1 $+ "," $+ point3;
         endif;
         text1 = text1 $+ " < " $+ name3 $+ ".h";
         text1 = text1 $+ " | qhull s Fa p Tv > " $+ name3 $+ ".asc";
         text3 = "limedt1.exe " $+ name3 $+ ".asc " $+ point1 $+ " " $+ point2;
         if dim==3;
           text3= text3 $+ " " $+ point3;
         endif;
         text1;
         text3;
         point[3]=point[3]+eta;
       endo;
       point[2]=point[2]+eta;
     endo;
     point[1]=point[1]+eta;
   endo;
   " ";
   text1 = "limedt2.exe " $+ name3 $+ ".ass";
   text1;
   text1 = strsect(name3,1,7);
   text1 = text1 $+ "_.bat";
   text1;
   format /rd 5,2;
   screen on;
   output off;

return;





/*
** orient - gives orientation of point with respect to hyperplane
*/

proc orient(H,x);

   local a,b,c,dis,sign;
   a=H[.,1];
   b=H[.,2];
   c=H[.,3];
   dis = det((b-a)~(c-a)~(x-a));
   sign = (dis>=0)-(dis<=0);
   retp (sign);

endp;





/*
** hesse - calculates normal vector of hyperplane
*/

proc(2) = hesse(H);

   local a,b,n,p,hn;
   a = H[.,1] - H[.,3];
   b = H[.,2] - H[.,3];
   n = {0,0,0};
   n[1] = a[2]*b[3] - a[3]*b[2];
   n[2] = a[3]*b[1] - a[1]*b[3];
   n[3] = a[1]*b[2] - a[2]*b[1];
   hn = (-1) .* n ./ (sqrt(n'*n));
   p = (-1) .* (hn' * H[.,3]);
   retp (hn,p);

endp;





/*
** lire0 - counts unicameral weighted votes on hyperplane H, in H+ and in H-
*/

proc lire0(H,x,g);

   local i,z,zz;
   zz=zeros(3,gew);
   i=0;
   do until i==m;
      i=i+1;
      z=orient(H,x[i,.]');
      zz[z+2,.]=zz[z+2,.]+g[i,.];
   endo;
   retp(zz);

endp;





/*
** lire1 - counts k-cameral unweighted votes on hyperplane H, in H+ and in H-
*/

proc lire1(H,x,g,k);

   local ii,z,zz;
   zz=zeros(3,kcameral);
   ii=0;
   do until ii==m;
      ii=ii+1;
      z=orient(H,x[ii,.]');
      zz[z+2,k[ii]]=zz[z+2,k[ii]]+g[ii];
   endo;
   retp(zz);

endp;




/*
** bm2 - limiting median hyperplane for two dimensions
*/

proc bm2(x1,x2,g1,g2,number,th,t,typ);

   local z,zli,zre;
   if typ==0;
     z=0;
     @ votes in H- and on H @
     zli=number[1,.]+number[2,.];
     @ votes in H+ and on H @
     zre=number[3,.]+number[2,.];
     if ( (zli>=th) AND ( (NOT(zli-g1>=th)) AND (NOT(zli-g2>=th)) ) );
       z=-1;
     endif;
     if ( (zre>=th) AND ( (NOT(zre-g1>=th)) AND (NOT(zre-g2>=th)) ) );
       if z==-1;
         z=9;
       else;
         z=1;
       endif;
     endif;
     retp(z);
   else;
     z=0;
     @ votes in H- and on H @
     zli=number[1,.]+number[2,.];
     @ votes in H+ and on H @
     zre=number[3,.]+number[2,.];
     if ( (zli>=th AND sumc(zli')>=t) AND
        ( (NOT(zli-g1>=th) OR NOT(sumc(zli'-g1')>=t)) AND
        (NOT(zli-g2>=th) OR NOT(sumc(zli'-g2')>=t)) ) );
       z=-1;
     endif;
     if ( (zre>=th AND sumc(zre')>=t) AND
        ( (NOT(zre-g1>=th) OR NOT(sumc(zre'-g1')>=t)) AND
        (NOT(zre-g2>=th) OR NOT(sumc(zre'-g2')>=t)) ) );
       if z==-1;
         z=9;
       else;
         z=1;
       endif;
     endif;
     retp(z);
   endif;

endp;




/*
** bm3 - limiting median hyperplane for three dimensions
*/

proc bm3(x1,x2,x3,g1,g2,g3,number,th,t,typ);

   local z,zli,zre;
   if typ==0;
     z=0;
     @ votes in H- and on H @
     zli=number[1,.]+number[2,.];
     @ votes in H+ and on H @
     zre=number[3,.]+number[2,.];
     if ( (zli>=th) AND ( (NOT(zli-g1>=th)) AND
        (NOT(zli-g2>=th)) AND (NOT(zli-g3>=th)) ) );
       z=-1;
     endif;
     if ( (zre>=th) AND ( (NOT(zre-g1>=th)) AND
        (NOT(zre-g2>=th)) AND (NOT(zre-g3>=th)) ) );
       if z==-1;
         z=9;
       else;
         z=1;
       endif;
     endif;
     retp(z);
   else;
     z=0;
     @ votes in H- and on H @
     zli=number[1,.]+number[2,.];
     @ votes in H+ and on H @
     zre=number[3,.]+number[2,.];
     if ( (zli>=th AND sumc(zli')>=t) AND
        ( (NOT(zli-g1>=th) OR NOT(sumc(zli'-g1')>=t)) AND
        (NOT(zli-g2>=th) OR NOT(sumc(zli'-g2')>=t)) AND
        (NOT(zli-g3>=th) OR NOT(sumc(zli'-g3')>=t)) ) );
       z=-1;
     endif;
     if ( (zre>=th AND sumc(zre')>=t) AND
        ( (NOT(zre-g1>=th) OR NOT(sumc(zre'-g1')>=t)) AND
        (NOT(zre-g2>=th) OR NOT(sumc(zre'-g2')>=t)) AND
        (NOT(zre-g3>=th) OR NOT(sumc(zre'-g3')>=t)) ) );
       if z==-1;
         z=9;
       else;
         z=1;
       endif;
     endif;
     retp(z);
   endif;

endp;





/*
**
** main
**
*/

mainproc:

   cls;
   gosub outlog;
   "LIMED - Limiting median hyperplanes for n-dimensional k-cameral "
   "legislatures";
   "(c) 2002 Thomas Braeuninger, University of Konstanz, Germany";
   " ";
   gosub loaddat;
   "processing...";
   " ";
   Hplane = zeros(1,dim1+1);

   if dim==2;
    "limiting median hyperplanes";
    "(2-point form, vector-offset form)";
    i=0;
    do until i==m;
      i=i+1;
      ii=i;
      do until ii==m;
        ii=ii+1;
        @ select two points a and c, b is arbitrary @
        a=x[i,.]';
        b=x[i,.]'; b[3]=1;
        c=x[ii,.]';
        v=a-c;
        w=b-c;
        H=a~b~c;
        if typ==0;
          number=lire0(H,x,g);
        else;
          number=lire1(H,x,g,k);
        endif;
        if typ==0;
          g1=g[i,.];
          g2=g[ii,.];
        else;
          g1=zeros(1,kcameral); g1[k[i]]=g[i];
          g2=zeros(1,kcameral); g2[k[ii]]=g[ii];
        endif;
        mm=bm2(a,c,g1,g2,number,thres,T,typ);
        lh = {0,0,0};
        lh[1] = v[2]*w[3] - v[3]*w[2];
        lh[2] = v[3]*w[1] - v[1]*w[3];
        lh[3] = v[1]*w[2] - v[2]*w[1];
        test=sumc(abs(lh));
        if (test>0) AND (mm/=0);
          if (mm==1) OR (mm==-1);
            {vekt,offset}=hesse(H);
            offset=mm.*offset;
            vekt=mm.*vekt;
            Hplane = Hplane | (vekt'~offset);
            i~ii~(vekt[1:2])'~offset;
          endif;
          if (mm==9);
            "Median hyperplane is attractive both ways!";
            {vekt,offset}=hesse(H);
            i~ii~(vekt[1:2])'~offset;
            "and";
            i~ii~((-1).*(vekt[1:2])')~((-1).*offset);
            "Replace by two hyperplanes in distance 2*eta? (y/n) ";
            replac = cons;
            if (lower(replac) $== "y");
              i~ii~(vekt[1:2])'~(offset-2*eta);
              Hplane = Hplane | (vekt'~(offset-2*eta));
              i~ii~((-1).*(vekt[1:2])')~((-1).*offset-2*eta);
              Hplane = Hplane | (((-1).*vekt')~((-1).*offset-2*eta));
            else;
              i~ii~(vekt[1:2])'~offset;
              Hplane = Hplane | (vekt'~offset);
            endif;
          endif;
        endif;
      endo;
    endo;
   endif;

   if dim==3;
    "limiting median hyperplanes";
    "(3-point form, vector-offset form)";
    i=0;
    do until i==m;
     i=i+1;
     ii=i;
     do until ii==m;
       ii=ii+1;
       iii=ii;
       do until iii==m;
         iii=iii+1;
         @ select three points a, b and c @
         a=x[i,.]';
         b=x[ii,.]';
         c=x[iii,.]';
         v=a-c;
         w=b-c;
         H=a~b~c;
         if typ==0;
           number=lire0(H,x,g);
         else;
           number=lire1(H,x,g,k);
         endif;
         if typ==0;
           g1=g[i,.];  g2=g[ii,.];  g3=g[iii,.];
         else;
           g1=zeros(1,kcameral); g1[k[i]]=g[i];
           g2=zeros(1,kcameral); g2[k[ii]]=g[ii];
           g3=zeros(1,kcameral); g3[k[iii]]=g[iii];
         endif;
         mm=bm3(a,b,c,g1,g2,g3,number,thres,T,typ);
         lh = {0,0,0};
         lh[1] = v[2]*w[3] - v[3]*w[2];
         lh[2] = v[3]*w[1] - v[1]*w[3];
         lh[3] = v[1]*w[2] - v[2]*w[1];
         test=sumc(abs(lh));
         if (test>0) AND (mm/=0);
           if (mm==1) OR (mm==-1);
             {vekt,offset}=hesse(H);
             offset=mm.*offset;
             vekt=mm.*vekt;
             Hplane = Hplane | (vekt'~offset);
             i~ii~iii~vekt'~offset;
           endif;
           if (mm==9);
             "Median hyperplane is attractive both ways!";
             {vekt,offset}=hesse(H);
             i~ii~iii~vekt'~offset;
             "  and";
             i~ii~iii~((-1).*vekt')~offset;
             "Replace by two hyperplanes in distance 2*eta? (y/n) ";
             replac = cons;
             if (lower(replac) $== "y");
               i~ii~iii~vekt'~(offset+2*eta);
               Hplane = Hplane | (vekt'~(offset+2*eta));
               i~ii~iii~((-1).*vekt')~(offset-2*eta);
               Hplane = Hplane | (((-1).*vekt')~(offset-2*eta));
             else;
               i~ii~iii~vekt'~offset;
               Hplane = Hplane | (vekt'~offset);
             endif;
           endif;
         endif;
       endo;
     endo;
   endo;
  endif;

  " ";
  "log end";
  output off;
  name31 = "del " $+ name3 $+ ".log";
  dos ^name31;
  name31 = "rename " $+ name5 $+ " " $+ name3 $+ ".log";
  dos ^name31;

  if (lower(qhull) $== "y");
    screen off;
    name32 = "del " $+ name3 $+ ".m";
    dos ^name32;
    name32 = "del " $+ name3 $+ ".txt";
    dos ^name32;
    name32 = "del " $+ name3 $+ ".h";
    dos ^name32;
    name32 = "del " $+ name3 $+ ".ass";
    dos ^name32;
    name32 = "del " $+ name3 $+ ".asc";
    dos ^name32;
    screen on;
    gosub outqhull;
  endif;

end;

